package tools.descartes.dql.connector.kieker;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Map.Entry;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import edu.kit.ipd.descartes.solving.ValidationError;
import tools.descartes.dql.core.engine.util.DQLLogger;
import tools.descartes.dql.models.mapping.domain.Service;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;

/* 
 * Helper class for various tasks and property store for Palladio
 */
public class KiekerHelper {


	public static final String METRIC_UTILIZATION = "utilization";
	public static final String METRIC_RESPONSE_TIME_AVG = "avgResponseTime";
	public static final String METRIC_THROUGHPUT = "throughput";

	private static final Logger log = DQLLogger.getLogger(KiekerHelper.class.getName());

	public static void checkProperties(final Properties properties,
			final String[] keys) {
		for (final String key : keys) {
			if (properties.get(key) == null) {
				final String msg = "Required key in properties missing: " + key;

				log.error(msg);
				throw new RuntimeException(msg);
			}
		}
	}







	public static void openValidationErrorWindow(Collection<ValidationError> errors) {
		final List<ValidationError> finalErrors = new LinkedList<ValidationError>();
		finalErrors.addAll(errors);
		
		PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
		    public void run() {
			    Shell activeShell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
				MessageDialog dialog = new ValidationErrorDialog(activeShell, finalErrors);
				dialog.open();
			}
		});
	}


	public static Properties getAbsolutePathProperties(final String kiekerPropertiesLocation) {
		final Properties properties = new Properties();
		File file = new File(kiekerPropertiesLocation);

		if (file == null || !file.exists()) {
			log.error("Specified file in USING not accessible, unable to execute the query further. Aborting.");
			return null;
		}

		try {
			properties.load(new FileInputStream(file));
		} catch (final IOException e) {
			log.error("Could not load model instance properties from specified model location in USING clause");
			e.printStackTrace();
		}
		String propertiesFolder = new File(kiekerPropertiesLocation).getParentFile().getAbsolutePath();

		for (Entry<Object, Object> property : properties.entrySet()) {
			String path = (String) property.getValue();
			if (!new File(path).exists()) {
				property.setValue(propertiesFolder + File.separator + path);
			}
		}

		return properties;
	}

	public static ArrayList<String> getServiceIdentifiers(EntityMapping request) {
		ArrayList<String> services = new ArrayList<>();
		for (Service service : request.getServices()) {
			services.add(service.getIdentifier());
		}
		return services;
	}

}
